function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var react = require('react');
var react__default = _interopDefault(react);

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var acceptedValidators = {
  required: true,
  email: true,
  maxLength: true,
  minLength: true
};
var regexAcceptedValidators = {
  pattern: true,
  validMobile: true,
  validBirthDate: true,
  passwordStrength: true
};

var splitAString = function splitAString(validatorString, op) {
  return validatorString.split(op);
};
var removeInvalidAndSplit = function removeInvalidAndSplit(validatorStringArray, op) {
  var validatorsAfterRemoval = validatorStringArray.map(function (key) {
    return key.includes(op) ? key.split(op) : [key, true];
  }).filter(function (keyArray) {
    if (acceptedValidators[keyArray[0]] === undefined || keyArray.length > 2) {
      return false;
    }

    return true;
  });
  return validatorsAfterRemoval;
};
var removeDuplicates = function removeDuplicates(validatorStringArray) {
  var keysLength = validatorStringArray.length;
  return validatorStringArray.filter(function (key, index) {
    var keyFieldValue = key[0];

    for (var itr = index + 1; itr < keysLength; itr++) {
      var iteratedFieldValue = validatorStringArray[itr][0];

      if (iteratedFieldValue.includes(keyFieldValue) || keyFieldValue.includes(iteratedFieldValue)) {
        return false;
      }
    }

    return true;
  });
};
var checkValuesValidity = function checkValuesValidity(validatorStringArray, type) {
  return validatorStringArray.filter(function (item) {
    if (item[0] === 'required') return item[1] === true ? true : false;
    if (item[0] === 'email') return item[1] === true ? true : false;
    if (item[0] === 'maxLength') return Number.isInteger(Number(item[1])) ? true : false;
    if (item[0] === 'minLength') return Number.isInteger(Number(item[1])) ? true : false;
    if (item[0] === 'pattern') return item[1] instanceof RegExp ? true : false;

    if (item[0] === 'validMobile') {
      if (item[1] instanceof RegExp) return true;
      if (item[1] === true) return true;
      return false;
    }

    if (item[0] === 'passwordStrength') {
      if (item[1] === true) return true;

      if (typeof item[1] === 'string') {
        var acceptedKeys = ['Lc', 'Uc', 'L', 'D', 'S'];
        var splitKeys = splitAString(item[1], '|');
        splitKeys = splitKeys.filter(function (key) {
          var keyFields = splitAString(key, ':');
          if (keyFields.length !== 2) return true;
          if (acceptedKeys.includes(keyFields[0]) && Number.isInteger(Number(keyFields[1]))) return false;
          return true;
        });
        if (splitKeys.length === 0) return true;
      }

      return false;
    }

    if (item[0] === 'validBirthDate') {
      if (item[1] === true) return true;
      if (typeof item[1] === 'string' && type !== 'date' && item[1].split('-').length === 3 && item[1].length === 10 && item[1].includes('mm') && item[1].includes('dd') && item[1].includes('yyyy')) return true;
      return false;
    }
  });
};
var mapRegexValidators = function mapRegexValidators(regexValidators) {
  var regexValidatorKeys = Object.keys(regexValidators);
  return regexValidatorKeys.filter(function (key) {
    return regexAcceptedValidators[key] === undefined ? false : true;
  }).map(function (key) {
    return [key, regexValidators[key]];
  });
};
var mapArrayToObj = function mapArrayToObj(validatorArray) {
  return validatorArray.map(function (key) {
    var _ref;

    return _ref = {}, _ref[key[0]] = key[1], _ref;
  });
};

var testForCustomRegex = function testForCustomRegex(regex, value) {
  var regexPattern = new RegExp(regex);
  return regexPattern.test(value) ? true : false;
};
var testForRegex = function testForRegex(regex, value) {
  return regex.test(value) ? true : false;
};
var testForMaxLength = function testForMaxLength(actualLength, requiredLength) {
  return actualLength <= requiredLength;
};
var testForMinLength = function testForMinLength(actualLength, requiredLength) {
  return actualLength >= requiredLength;
};
var testForDOBInputType = function testForDOBInputType(dobArr) {
  var truthyArray = dobArr.filter(function (elem) {
    return Number.isInteger(Number(elem));
  });
  return truthyArray.length < 3 ? false : true;
};
var testForValidDOB = function testForValidDOB(dobArr) {
  var birthDate = Number(dobArr[0]);
  var birthMonth = Number(dobArr[1]);
  var birthYear = Number(dobArr[2]);
  var inValidDate = false;
  var isLeapyear = false;
  var month31 = [1, 3, 5, 7, 8, 10, 12];
  var month30 = [4, 6, 9, 11];

  if (birthYear >= 1950 && birthYear <= new Date().getFullYear() && Number.isInteger(birthDate) && Number.isInteger(birthMonth) && Number.isInteger(birthYear)) {
    if (birthYear % 4 == 0 && birthYear % 100 != 0 || birthYear % 400 == 0) {
      isLeapyear = true;
    }

    if (birthYear === new Date().getFullYear()) {
      if (birthDate > new Date().getDate() && birthMonth >= new Date().getMonth() + 1) {
        inValidDate = true;
      } else if (birthMonth > new Date().getMonth() + 1) {
        inValidDate = true;
      }
    }

    if (month31.includes(birthMonth)) {
      if (birthDate < 1 || birthDate > 31) inValidDate = true;
    } else if (month30.includes(birthMonth)) {
      if (birthDate < 1 || birthDate > 30) inValidDate = true;
    } else if (birthMonth === 2 && isLeapyear) {
      if (birthDate < 1 || birthDate > 29) inValidDate = true;
    } else if (birthMonth === 2 && !isLeapyear) {
      if (birthDate < 1 || birthDate > 28) inValidDate = true;
    } else {
      inValidDate = true;
    }
  } else {
    inValidDate = true;
  }

  return inValidDate;
};
var composeRulesAndValidate = function composeRulesAndValidate(rulesStr, val) {
  var rulesSplitArray = splitAString(rulesStr, '|');
  var validationResultArr = rulesSplitArray.map(function (rule) {
    var fieldvalues = splitAString(rule, ':');

    if (fieldvalues[0] === 'Uc') {
      var _ref;

      return _ref = {}, _ref['hasUpperCase'] = !testForRegex(new RegExp("^(.*[A-Z]){" + fieldvalues[1] + "}.*$"), val), _ref;
    } else if (fieldvalues[0] === 'Lc') {
      var _ref2;

      return _ref2 = {}, _ref2['hasLowerCase'] = !testForRegex(new RegExp("^(.*[a-z]){" + fieldvalues[1] + "}.*$"), val), _ref2;
    } else if (fieldvalues[0] === 'D') {
      var _ref3;

      return _ref3 = {}, _ref3['hasNumbers'] = !testForRegex(new RegExp("^(.*[0-9]){" + fieldvalues[1] + "}.*$"), val), _ref3;
    } else if (fieldvalues[0] === 'S') {
      var _ref4;

      return _ref4 = {}, _ref4['hasSymbols'] = !testForRegex(new RegExp("^(.*[*!@#$&*]){" + fieldvalues[1] + "}.*$"), val), _ref4;
    } else if (fieldvalues[0] === 'L') {
      var _ref5;

      return _ref5 = {}, _ref5['hasMinimumLength'] = !testForMinLength(val.length, Number(fieldvalues[1])), _ref5;
    }
  });
  return Object.assign.apply(Object, [{}].concat(validationResultArr));
};

var validations = function validations(fieldValue, validationsToPerform, type) {
  var errorsOccurred = {};
  var emailRegex = /[a-z0-9!#$%&'*+=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?/;
  var actualLength = String(fieldValue).length;
  var defaultMobileRegex = /^[6-9]\d{9}$/;
  var defaultDateRegex = /^(0?[1-9]|[12][0-9]|3[01])[\/\-](0?[1-9]|1[012])[\/\-]\d{4}$/;
  var defaultPasswordString = 'Lc:2|Uc:3|D:2|S:1|L:8';

  if (validationsToPerform.required && !fieldValue) {
    errorsOccurred.required = true;
  }

  if (validationsToPerform.email && fieldValue && !testForRegex(emailRegex, fieldValue)) {
    errorsOccurred.email = true;
  }

  if (validationsToPerform.maxLength && fieldValue && !testForMaxLength(actualLength, +validationsToPerform.maxLength)) {
    errorsOccurred.maxLength = {
      actualLength: actualLength,
      requiredLength: +validationsToPerform.maxLength
    };
  }

  if (validationsToPerform.minLength && fieldValue && !testForMinLength(actualLength, +validationsToPerform.minLength)) {
    errorsOccurred.minLength = {
      actualLength: actualLength,
      requiredLength: +validationsToPerform.minLength
    };
  }

  if (validationsToPerform.pattern && fieldValue && !testForCustomRegex(validationsToPerform.pattern, fieldValue)) {
    errorsOccurred.pattern = true;
  }

  if (validationsToPerform.validMobile && fieldValue) {
    if (validationsToPerform.validMobile === true) {
      if (Number.isInteger(Number(fieldValue))) {
        if (!testForRegex(defaultMobileRegex, fieldValue)) errorsOccurred.validMobile = true;
      } else {
        errorsOccurred.validMobile = true;
      }
    } else if (validationsToPerform.validMobile instanceof RegExp && !testForCustomRegex(validationsToPerform.validMobile, fieldValue)) {
      errorsOccurred.validMobile = true;
    }
  }

  if (validationsToPerform.validBirthDate && fieldValue) {
    var dateToBePassed = '';
    var dateSplitArray = [];
    var fieldvalueArray = [];
    var fieldValueToBePassed = fieldValue;
    var canTriggerValidation = false;

    if (type === 'date') {
      var dateSplit = fieldValue.split('-');
      fieldvalueArray = [dateSplit[2], dateSplit[1], dateSplit[0]];
      fieldValueToBePassed = dateSplit[2] + "/" + dateSplit[1] + "/" + dateSplit[0];
    } else {
      fieldvalueArray = fieldValue.split('/');
    }

    if (validationsToPerform.validBirthDate === true) {
      dateToBePassed = fieldValueToBePassed;
      dateSplitArray = [].concat(fieldvalueArray);
      canTriggerValidation = true;
    } else if (typeof validationsToPerform.validBirthDate === 'string' && type !== 'date') {
      var date = null;
      var month = null;
      var year = null;
      canTriggerValidation = true;
      var patternToPass = validationsToPerform.validBirthDate.split('-');
      patternToPass.forEach(function (val, pos) {
        if (val === 'dd') date = Number(fieldvalueArray[pos]);
        if (val === 'mm') month = Number(fieldvalueArray[pos]);
        if (val === 'yyyy') year = Number(fieldvalueArray[pos]);
      });
      dateToBePassed = date + "/" + month + "/" + year;
      dateSplitArray = [date, month, year];
    }

    if (canTriggerValidation) {
      if (!testForRegex(defaultDateRegex, dateToBePassed) || !testForDOBInputType(dateSplitArray) || testForValidDOB(dateSplitArray)) {
        errorsOccurred.validBirthDate = true;
      }
    }
  }

  if (validationsToPerform.passwordStrength && typeof fieldValue === 'string' && fieldValue) {
    var validationResultObj = {};

    if (typeof validationsToPerform.passwordStrength === 'string') {
      validationResultObj = composeRulesAndValidate(validationsToPerform.passwordStrength, fieldValue);
    } else if (validationsToPerform.passwordStrength === true) {
      validationResultObj = composeRulesAndValidate(defaultPasswordString, fieldValue);
    }

    var errorsIfAvailable = Object.keys(validationResultObj).filter(function (key) {
      return validationResultObj[key] ? true : false;
    }).length;

    if (errorsIfAvailable !== 0) {
      errorsOccurred.passwordStrength = _extends({
        invalid: true
      }, validationResultObj);
    }
  }

  return errorsOccurred;
};

var useFormsio = function useFormsio() {
  var _useState = react.useState({}),
      formState = _useState[0],
      setFormState = _useState[1];

  var refs = react.useRef({});
  var formStateRef = react.useRef({});
  var validationRules = react.useRef({});
  var isFormValid = react.useRef(false);
  var initialValues = {};
  var supportedFields = ['checkbox', 'radio', 'text', 'tel', 'password', 'email', 'number', 'date', 'file', 'range', 'url', 'select', 'textarea'];

  var handleChange = function handleChange(event) {
    var _event$target = event.target,
        name = _event$target.name,
        value = _event$target.value,
        type = _event$target.type,
        checked = _event$target.checked;

    var refFieldValues = _extends({}, formStateRef.current[name]);

    if (refFieldValues.pristine === false || refFieldValues.dirty === false) {
      formStateRef.current[name] = _extends({}, formStateRef.current[name], {
        pristine: false,
        dirty: true
      });
    }

    var fieldValue = value;
    if (type === 'checkbox') fieldValue = checked;
    setFormState(function (prevState) {
      var _extends2;

      var fieldValues = _extends({}, prevState[name]);

      var errorsAvailable = validations(fieldValue, validationRules.current[name], type);
      var haveErrors = Object.keys(errorsAvailable).length > 0;
      formStateRef.current[name] = _extends({}, formStateRef.current[name], {
        invalid: haveErrors,
        valid: !haveErrors
      });
      checkFormValidity();
      return _extends({}, prevState, (_extends2 = {}, _extends2[name] = _extends({}, fieldValues, {
        value: fieldValue,
        pristine: fieldValues.pristine ? !fieldValues.pristine : fieldValues.pristine,
        dirty: fieldValues.dirty ? fieldValues.dirty : !fieldValues.dirty,
        errors: _extends({}, errorsAvailable),
        invalid: haveErrors,
        valid: !haveErrors
      }), _extends2));
    });
  };

  var handleFocus = function handleFocus(event) {
    var name = event.target.name;

    var refFieldValues = _extends({}, formStateRef.current[name]);

    if (refFieldValues.touched === false || refFieldValues.untouched === true) {
      formStateRef.current[name] = _extends({}, formStateRef.current[name], {
        touched: true,
        untouched: false
      });
      setFormState(function (prevState) {
        var _extends3;

        var fieldValues = _extends({}, prevState[name]);

        return _extends({}, prevState, (_extends3 = {}, _extends3[name] = _extends({}, fieldValues, {
          touched: true,
          untouched: false
        }), _extends3));
      });
    }
  };

  var checkFormValidity = function checkFormValidity() {
    var formStateKeys = Object.keys(formStateRef.current);

    if (formStateKeys.length !== 0) {
      var truthValueArray = formStateKeys.filter(function (key) {
        return formStateRef.current[key].invalid ? true : false;
      });
      isFormValid.current = truthValueArray.length > 0 ? false : true;
    }
  };

  react.useEffect(function () {}, [formState]);

  var register = function register(fieldArgs) {
    return function (ref) {
      if (ref) {
        var type = '';
        var tagName = document.getElementsByName(ref.name)[0].tagName.toLowerCase();
        type = tagName === 'select' || tagName === 'textarea' ? tagName : ref.type;

        if (supportedFields.includes(type)) {
          var refName = type === 'radio' ? ref.id : ref.name;

          if (refs.current[refName] === undefined) {
            var validators = (fieldArgs === null || fieldArgs === void 0 ? void 0 : fieldArgs.validators) ? fieldArgs === null || fieldArgs === void 0 ? void 0 : fieldArgs.validators : null;
            var regexValidators = (fieldArgs === null || fieldArgs === void 0 ? void 0 : fieldArgs.regexValidators) ? fieldArgs === null || fieldArgs === void 0 ? void 0 : fieldArgs.regexValidators : null;
            var initialValue = (fieldArgs === null || fieldArgs === void 0 ? void 0 : fieldArgs.initialValue) ? fieldArgs === null || fieldArgs === void 0 ? void 0 : fieldArgs.initialValue : '';
            var objAfterComposed = validators || regexValidators ? composeValidators(validators, regexValidators, type) : {};

            if (type === 'checkbox' || type === 'radio' || type === 'select' || type === 'file' || type === 'range') {
              objAfterComposed = Object.keys(objAfterComposed).reduce(function (acc, val) {
                if (val === 'required') {
                  var _extends4;

                  return _extends({}, acc, (_extends4 = {}, _extends4[val] = objAfterComposed[val], _extends4));
                } else return acc;
              }, {});
            }

            var name = ref.name;
            validationRules.current[name] = objAfterComposed;
            if (type === 'radio') name = ref.id;
            initialValues[name] = initialValue ? initialValue : '';
            refs.current[name] = ref;
          }
        }
      }
    };
  };

  var composeValidators = function composeValidators(validatorStr, regexValidators, type) {
    var validatorRules = [];
    var regexValidatorRules = [];

    if (type) {
      if (validatorStr) {
        var validatorRulesSplitArray = removeInvalidAndSplit(splitAString(validatorStr, '|'), ':');
        validatorRules = mapArrayToObj(removeDuplicates(checkValuesValidity(validatorRulesSplitArray, type)));
      } else {
        validatorRules = [];
      }

      if (regexValidators) {
        regexValidatorRules = mapArrayToObj(checkValuesValidity(mapRegexValidators(regexValidators), type));
      } else {
        regexValidatorRules = [];
      }
    }

    return Object.assign.apply(Object, [{}].concat(validatorRules.concat(regexValidatorRules)));
  };

  react.useEffect(function () {
    var refsKeys = Object.keys(refs.current);
    refsKeys.forEach(function (refKey) {
      refs.current[refKey].value = initialValues[refKey];
      refs.current[refKey].oninput = handleChange;
      refs.current[refKey].onfocus = handleFocus;
      var fieldName = refs.current[refKey].name;

      if (formStateRef.current && (formStateRef.current[refKey] === undefined && formStateRef.current[fieldName] === undefined || refs.current[refKey].type === 'radio')) {
        var fieldValues = {
          value: initialValues[refKey],
          untouched: true,
          touched: false,
          pristine: true,
          dirty: false,
          errors: {}
        };
        var refCompoundName = refKey;
        var radioChild = false;
        var newRadio = false;
        var otherThanRadio = false;

        if (refs.current[refKey].type === 'radio') {
          refCompoundName = fieldName;
          if (formStateRef.current[refCompoundName] != undefined) radioChild = true;else newRadio = true;
        } else {
          otherThanRadio = true;
        }

        var validityBasedOnValidators = Object.keys(validationRules.current[refCompoundName]).length > 0;
        formStateRef.current[refCompoundName] = {
          touched: false,
          untouched: true,
          pristine: true,
          dirty: false,
          invalid: validityBasedOnValidators,
          valid: !validityBasedOnValidators
        };
        setFormState(function (prevState) {
          if (radioChild === true) {
            var _extends5, _extends6;

            return _extends({}, prevState, (_extends6 = {}, _extends6[refCompoundName] = _extends({}, prevState[refCompoundName], {
              invalid: validityBasedOnValidators,
              valid: !validityBasedOnValidators,
              children: _extends({}, prevState[refCompoundName].children, (_extends5 = {}, _extends5[refKey] = true, _extends5))
            }), _extends6));
          } else if (newRadio === true) {
            var _children, _extends7;

            return _extends({}, prevState, (_extends7 = {}, _extends7[refCompoundName] = _extends({}, fieldValues, {
              invalid: validityBasedOnValidators,
              valid: !validityBasedOnValidators,
              value: '',
              children: (_children = {}, _children[refKey] = true, _children)
            }), _extends7));
          } else if (otherThanRadio === true) {
            var _extends8;

            return _extends({}, prevState, (_extends8 = {}, _extends8[refCompoundName] = _extends({}, fieldValues, {
              invalid: validityBasedOnValidators,
              valid: !validityBasedOnValidators
            }), _extends8));
          }
        });
      }
    });
  }, []);
  return {
    register: register,
    formState: formState,
    validationRules: validationRules.current,
    isFormValid: isFormValid.current
  };
};

exports.useFormsio = useFormsio;
//# sourceMappingURL=index.js.map
